import requests, os

def download_github_dir(owner, repo, path, branch="master", save_dir=".", flat=False):
    """Download a GitHub folder recursively."""
    url = f"https://api.github.com/repos/{owner}/{repo}/contents/{path}?ref={branch}"
    response = requests.get(url).json()

    if isinstance(response, dict) and response.get("message"):
        raise Exception(f"GitHub API error: {response['message']}")

    for item in response:
        if item["type"] == "file":
            if flat:
                # Save only filename in current dir
                item_path = os.path.join(save_dir, os.path.basename(item["path"]))
            else:
                # Keep directory structure
                item_path = os.path.join(save_dir, item["path"])
                os.makedirs(os.path.dirname(item_path), exist_ok=True)

            print(f"⬇️  Downloading {item['download_url']}")
            file_data = requests.get(item["download_url"]).content
            with open(item_path, "wb") as f:
                f.write(file_data)

        elif item["type"] == "dir":
            download_github_dir(owner, repo, item["path"], branch, save_dir, flat)




import sys
from .core import download_github_dir
from .help import show_help

def main():
    if len(sys.argv) < 2:
        show_help()
        sys.exit(1)

    flat = False
    save_dir = "."

    # size of argv in 3 or more -- command has flags
    if sys.argv[1].startswith(("-","--")):
        # size of argv is 3 or 2 if -h or --help 
        if sys.argv[1] == "-nf":
            flat = True
        elif (sys.argv[1] == "--help") or (sys.argv[1] == "-h"):
            show_help()
            sys.exit(1)
        else:
            print("Unknown flag:", sys.argv[1])
            sys.exit(1)
        if len(sys.argv) == 2:
            print("No GitHub url found\n\ngiget -nf <github-url>")
            sys.exit(1)
        url = sys.argv[2].rstrip("/")

        # checking if -o flag is present -- if yes then size of argv should be 5
        if len(sys.argv) >= 4:
            
            if sys.argv[3] == "-o":
                if len(sys.argv == 4):
                    print("Missing output directory after -o")
                    sys.exit(1)
                save_dir = sys.argv[4]
            else:
                print("Unknown flag:", sys.argv[3])
                sys.exit(1)
    
    # size of argv is 2 -- a normal CLI command
    else:
        url = sys.argv[1].rstrip("/")
        # checking if -o is present
        if len(sys.argv) > 3:
            if sys.argv[2] == "-o":
                if len(sys.argv == 3):
                    print("Missing output directory after -o")
                    sys.exit(1)
                save_dir = sys.argv[3]
            else:
                print("Unknown flag:", sys.argv[3])
                sys.exit(1)

    try:
        # Example: https://github.com/jasmcaus/opencv-course/tree/master/Resources/Photos
        parts = url.split("github.com/")[1].split("/")
        owner, repo, tree, branch, *path = parts
        folder_path = "/".join(path)
    except Exception:
        print("****| Invalid GitHub URL format. |****")
        sys.exit(1)

    download_github_dir(owner, repo, folder_path, branch, save_dir=save_dir, flat=flat)
    print("✅ Download complete!")


# help.py

def show_help():
    help_text = """
giget - A simple CLI tool to download GitHub directories | sub-directories | files.

Usage:
    giget [flags] <github_repo_url> [options]

Options:
    -h, --help       Show this help message and exit.
    -o <path>        Specify output directory (default: current directory).
    -nf <path>       Downloads all files into current dir (no folders)

Examples:
    giget https://github.com/user/repo
    giget https://github.com/user/repo -o ./downloads
    giget -nf https://github.com/user/repo

Description:
    giget is a Python package and CLI tool that makes it easy to download
    any directories or all the files inside directories 
    from GitHub repositories.

    Steps:
      1. Provide a GitHub repository URL.
      2. giget will fetch and download the files.
      3. You can specify an output folder using -o.
      4. If you want to download all the files and dont want the 
         structure of the folder then use -nf.
"""
    print(help_text)
